from .. import config
from .. import serializers
from ..repositories.common import CreateResource, ListResources, DeleteResource, GetResource
from ..sdk_exceptions import ResourceFetchingError


class GetBaseProjectsApiUrlMixin(object):
    def _get_api_url(self, **_):
        return config.config.CONFIG_HOST


class CreateProject(GetBaseProjectsApiUrlMixin, CreateResource):
    SERIALIZER_CLS = serializers.Project

    def get_request_url(self, **_):
        return "/projects/"


class ListProjects(GetBaseProjectsApiUrlMixin, ListResources):
    SERIALIZER_CLS = serializers.Project

    def get_request_url(self, **kwargs):
        return "/projects/"

    def _get_instance_dicts(self, data, **kwargs):
        project_dict_list = data["data"]
        return project_dict_list

    def _get_request_params(self, kwargs):
        params = {
            "filter": """{"offset":0,"where":{"dtDeleted":null},"order":"dtCreated desc"}"""
        }

        tags = kwargs.get("tags")
        if tags:
            params["tagFilter"] = tags

        return params


class DeleteProject(GetBaseProjectsApiUrlMixin, DeleteResource):
    def get_request_url(self, **kwargs):
        return "/projects/{}/deleteProject".format(kwargs.get("id"))

    def _send_request(self, client, url, json_data=None):
        response = client.post(url, json=json_data)
        return response


class GetProject(GetBaseProjectsApiUrlMixin, GetResource):
    SERIALIZER_CLS = serializers.Project

    def get_request_url(self, **kwargs):
        return "/projects/"

    def _get_request_params(self, kwargs):
        id_ = kwargs["id"]
        params = {
            "filter": """{"where":{"handle":"%s"}}""" % id_
        }

        return params

    def _parse_object(self, instance_dict, **kwargs):
        try:
            instance_dict = instance_dict["data"][0]
        except IndexError:
            raise ResourceFetchingError("Project not found")

        instance_dict = super(GetProject, self)._parse_object(instance_dict, **kwargs)
        return instance_dict



import click

from gradient.api_sdk import constants
from gradient.cli import common
from gradient.cli.cli import cli
from gradient.cli.cli_types import ChoiceType, json_string
from gradient.cli.common import validate_comma_split_option, api_key_option, ClickGroup
from gradient.commands import notebooks
from gradient.commands.notebooks import GetNotebookMetricsCommand, ListNotebookMetricsCommand, StreamNotebookMetricsCommand


@cli.group("notebooks", help="Manage notebooks", cls=common.ClickGroup)
def notebooks_group():
    pass


@notebooks_group.group("tags", help="Manage notebook tags", cls=common.ClickGroup)
def notebook_tags():
    pass


@notebooks_group.group(name="metrics", help="Read notebook metrics", cls=common.ClickGroup)
def notebook_metrics():
    pass


@notebooks_group.command("create", help="Create new notebook")
@click.option(
    "--machineType",
    "machine_type",
    required=True,
    type=str,
    help="Virtual Machine type label e.g. P5000",
    cls=common.GradientOption,
)
@click.option(
    "--container",
    "container",
    required=True,
    type=str,
    help="Container name",
    cls=common.GradientOption,
)
@click.option(
    "--projectId",
    "project_id",
    required=True,
    type=str,
    help="Project ID",
    cls=common.GradientOption,
)
@click.option(
    "--clusterId",
    "cluster_id",
    type=str,
    help="Cluster ID",
    cls=common.GradientOption,
)
@click.option(
    "--name",
    "name",
    type=str,
    help="Notebook name",
    cls=common.GradientOption,
)
@click.option(
    "--registryUsername",
    "registry_username",
    type=str,
    help="Registry username",
    cls=common.GradientOption,
)
@click.option(
    "--registryPassword",
    "registry_password",
    type=str,
    help="Registry password",
    cls=common.GradientOption,
)
@click.option(
    "--command",
    "command",
    type=str,
    help="Command (executed as `/bin/sh -c 'YOUR COMMAND'`)",
    cls=common.GradientOption,
)
@click.option(
    "--containerUser",
    "container_user",
    type=str,
    help="Container user",
    cls=common.GradientOption,
)
@click.option(
    "--shutdownTimeout",
    "shutdown_timeout",
    help="Shutdown timeout in hours",
    type=int,
    cls=common.GradientOption,
)
@click.option(
    "--isPreemptible",
    "is_preemptible",
    help="Is preemptible",
    is_flag=True,
    type=bool,
    cls=common.GradientOption,
)
@click.option(
    "--isPublic",
    "is_public",
    help="Is publically viewable",
    is_flag=True,
    type=bool,
    cls=common.GradientOption,
)
@click.option(
    "--environment",
    "environment",
    type=json_string,
    help="Environmental variables",
    cls=common.GradientOption,
)
@click.option(
    "--workspace",
    "workspace",
    help="S3 url or git repository. Directory uploads are not yet supported",
    default="none",
    cls=common.GradientOption,
)
@click.option(
    "--workspaceRef",
    "workspace_ref",
    help="Git commit hash, branch name or tag",
    cls=common.GradientOption,
)
@click.option(
    "--workspaceUsername",
    "workspace_username",
    metavar="<username>",
    cls=common.GradientOption,
)
@click.option(
    "--workspacePassword",
    "workspace_password",
    help="Workspace password",
    cls=common.GradientOption,
)
@click.option(
    "--tag",
    "tags",
    multiple=True,
    help="One or many tags that you want to add to experiment",
    cls=common.GradientOption
)
@click.option(
    "--tags",
    "tags_comma",
    help="Separated by comma tags that you want add to experiment",
    cls=common.GradientOption
)
@common.api_key_option
@common.options_file
def create_notebook(api_key, options_file, **notebook):
    notebook["tags"] = validate_comma_split_option(notebook.pop("tags_comma"), notebook.pop("tags"))
    command = notebooks.CreateNotebookCommand(api_key=api_key)
    command.execute(**notebook)

@notebooks_group.command("start", help="Start notebook")
@click.option(
    "--id",
    "id",
    type=str,
    required=True,
    help="Notebook ID",
    cls=common.GradientOption,
)
@click.option(
    "--machineType",
    "machine_type",
    required=True,
    type=str,
    help="Virtual Machine type label e.g. P5000",
    cls=common.GradientOption,
)
@click.option(
    "--clusterId",
    "cluster_id",
    type=str,
    help="Cluster ID",
    cls=common.GradientOption,
)
@click.option(
    "--name",
    "name",
    type=str,
    help="Notebook name",
    cls=common.GradientOption,
)
@click.option(
    "--shutdownTimeout",
    "shutdown_timeout",
    help="Shutdown timeout in hours",
    type=int,
    cls=common.GradientOption,
)
@click.option(
    "--isPreemptible",
    "is_preemptible",
    help="Is preemptible",
    is_flag=True,
    type=bool,
    cls=common.GradientOption,
)
@click.option(
    "--tag",
    "tags",
    multiple=True,
    help="One or many tags that you want to add to experiment",
    cls=common.GradientOption
)
@click.option(
    "--tags",
    "tags_comma",
    help="Separated by comma tags that you want add to experiment",
    cls=common.GradientOption
)

@common.api_key_option
@common.options_file
def start_notebook(api_key, options_file, **notebook):
    notebook["tags"] = validate_comma_split_option(notebook.pop("tags_comma"), notebook.pop("tags"))
    command = notebooks.StartNotebookCommand(api_key=api_key)
    command.execute(**notebook)

@notebooks_group.command("fork", help="Fork existing notebook")
@click.option(
    "--id",
    "id_",
    required=True,
    help="Notebook ID",
    cls=common.GradientOption,
)
@click.option(
    "--projectId",
    "project_id",
    required=True,
    type=str,
    help="Project ID",
    cls=common.GradientOption,
)
@common.api_key_option
@common.options_file
def fork_notebook(id_, project_id, api_key, options_file):
    command = notebooks.ForkNotebookCommand(api_key=api_key)
    command.execute(id_=id_, project_id=project_id)


@notebooks_group.command("delete", help="Delete existing notebook")
@click.option(
    "--id",
    "id_",
    help="Notebook ID",
    cls=common.GradientOption,
)
@common.api_key_option
@common.options_file
def delete_notebook(id_, api_key, options_file):
    command = notebooks.DeleteNotebookCommand(api_key=api_key)
    command.execute(id_=id_)


@notebooks_group.command("list", help="List notebooks")
@click.option("--limit", "-l", "n_limit", default=20)
@click.option("--offset", "-o", "n_offset", default=0)
@click.option(
    "--tag",
    "tags",
    multiple=True,
    cls=common.GradientOption,
    help="Filter by tags. Multiple use"
)
@common.api_key_option
@common.options_file
def list_notebooks(n_limit, n_offset, tags, api_key, options_file):
    command = notebooks.ListNotebooksCommand(api_key=api_key)
    for notebook_str, next_iteration in command.execute(limit=n_limit, offset=n_offset, tags=tags):
        click.echo(notebook_str)
        if next_iteration:
            click.confirm("Do you want to continue?", abort=True)


@notebooks_group.command("details", help="Show notebook details")
@click.option(
    "--id",
    "id",
    help="Notebook ID",
    cls=common.GradientOption,
    required=True,
)
@common.api_key_option
@common.options_file
def show_notebook(id, api_key, options_file):
    command = notebooks.ShowNotebookDetailsCommand(api_key=api_key)
    command.execute(id)


@notebook_tags.command("add", help="Add tags to notebook")
@click.option(
    "--id",
    "id",
    required=True,
    cls=common.GradientOption,
    help="ID of the notebook",
)
@click.option(
    "--tag",
    "tags",
    multiple=True,
    help="One or many tags that you want to add to notebook",
    cls=common.GradientOption
)
@click.option(
    "--tags",
    "tags_comma",
    help="Separated by comma tags that you want add to notebook",
    cls=common.GradientOption
)
@common.api_key_option
@common.options_file
def notebook_add_tag(id, options_file, api_key, **kwargs):
    kwargs["tags"] = validate_comma_split_option(kwargs.pop("tags_comma"), kwargs.pop("tags"), raise_if_no_values=True)

    command = notebooks.NotebookAddTagsCommand(api_key=api_key)
    command.execute(id, **kwargs)


@notebook_tags.command("remove", help="Remove tags from notebook")
@click.option(
    "--id",
    "id",
    required=True,
    cls=common.GradientOption,
    help="ID of the model",
)
@click.option(
    "--tag",
    "tags",
    multiple=True,
    help="One or many tags that you want to remove from notebook",
    cls=common.GradientOption
)
@click.option(
    "--tags",
    "tags_comma",
    help="Separated by comma tags that you want to remove from notebook",
    cls=common.GradientOption
)
@common.api_key_option
@common.options_file
def notebook_remove_tags(id, options_file, api_key, **kwargs):
    kwargs["tags"] = validate_comma_split_option(kwargs.pop("tags_comma"), kwargs.pop("tags"), raise_if_no_values=True)

    command = notebooks.NotebookRemoveTagsCommand(api_key=api_key)
    command.execute(id, **kwargs)


@notebook_metrics.command(
    "get",
    short_help="Get notebook metrics",
    help="Get notebook metrics. Shows CPU and RAM usage by default",
)
@click.option(
    "--id",
    "notebook_id",
    required=True,
    cls=common.GradientOption,
    help="ID of the notebook",
)
@click.option(
    "--metric",
    "metrics_list",
    multiple=True,
    type=str,
    default=(constants.BuiltinMetrics.cpu_percentage, constants.BuiltinMetrics.memory_usage),
    help=("One or more metrics that you want to read: {}. Defaults to cpuPercentage and memoryUsage. To view available custom metrics, use command: `gradient notebooks metrics list`".format(', '.join(map(str, constants.METRICS_MAP)))),
    cls=common.GradientOption,
)
@click.option(
    "--interval",
    "interval",
    default="30s",
    help="Interval",
    cls=common.GradientOption,
)
@click.option(
    "--start",
    "start",
    type=click.DateTime(),
    help="Timestamp of first time series metric to collect",
    cls=common.GradientOption,
)
@click.option(
    "--end",
    "end",
    type=click.DateTime(),
    help="Timestamp of last time series metric to collect",
    cls=common.GradientOption,
)
@api_key_option
@common.options_file
def get_deployment_metrics(notebook_id, metrics_list, interval, start, end, options_file, api_key):
    command = GetNotebookMetricsCommand(api_key=api_key)
    command.execute(notebook_id, start, end, interval, built_in_metrics=metrics_list)


@notebook_metrics.command(
    "list",
    short_help="List notebook metrics",
    help="List notebook metrics. Shows CPU and RAM usage by default",
)
@click.option(
    "--id",
    "notebook_id",
    required=True,
    cls=common.GradientOption,
    help="ID of the notebook",
)
@click.option(
    "--interval",
    "interval",
    default="30s",
    help="Interval",
    cls=common.GradientOption,
)
@click.option(
    "--start",
    "start",
    type=click.DateTime(),
    help="Timestamp of first time series metric to collect",
    cls=common.GradientOption,
)
@click.option(
    "--end",
    "end",
    type=click.DateTime(),
    help="Timestamp of last time series metric to collect",
    cls=common.GradientOption,
)
@api_key_option
@common.options_file
def list_deployment_metrics(notebook_id, interval, start, end, options_file, api_key):
    command = ListNotebookMetricsCommand(api_key=api_key)
    command.execute(notebook_id, start, end, interval)


@notebook_metrics.command(
    "stream",
    short_help="Watch live notebook metrics",
    help="Watch live notebook metrics. Shows CPU and RAM usage by default",
)
@click.option(
    "--id",
    "notebook_id",
    required=True,
    cls=common.GradientOption,
    help="ID of the notebook",
)
@click.option(
    "--metric",
    "metrics_list",
    multiple=True,
    type=ChoiceType(constants.METRICS_MAP, case_sensitive=False),
    default=(constants.BuiltinMetrics.cpu_percentage, constants.BuiltinMetrics.memory_usage),
    help="One or more metrics that you want to read. Defaults to cpuPercentage and memoryUsage",
    cls=common.GradientOption,
)
@click.option(
    "--interval",
    "interval",
    default="30s",
    help="Interval",
    cls=common.GradientOption,
)
@api_key_option
@common.options_file
def stream_model_deployment_metrics(notebook_id, metrics_list, interval, options_file, api_key):
    command = StreamNotebookMetricsCommand(api_key=api_key)
    command.execute(notebook_id=notebook_id, interval=interval, built_in_metrics=metrics_list)

@notebooks_group.command("stop", help="Stop running notebook")
@click.option(
    "--id",
    "notebook_id",
    required=True,
    help="Stop notebook with given ID",
)
@api_key_option
def stop_notebook(notebook_id, api_key=None):
    command = notebooks.StopNotebookCommand(api_key=api_key)
    command.execute(notebook_id)


@notebooks_group.group("artifacts", help="Manage notebooks' artifacts", cls=ClickGroup)
def artifacts():
    pass


@artifacts.command("list", help="List notebook's artifacts")
@click.option(
    "--id",
    "notebook_id",
    required=True,
    cls=common.GradientOption,
    help="ID of the notebook",
)
@click.option(
    "--size",
    "-s",
    "size",
    help="Show file size",
    is_flag=True,
    cls=common.GradientOption,
)
@click.option(
    "--links",
    "-l",
    "links",
    help="Show file URL",
    is_flag=True,
    default=False,
    cls=common.GradientOption,
)
@click.option(
    "--files",
    "files",
    help="Get only given file (use at the end * as a wildcard)",
    cls=common.GradientOption,
)
@api_key_option
@common.options_file
def list_artifacts(notebook_id, size, links, files, options_file, api_key=None):
    command = notebooks.ArtifactsListCommand(api_key=api_key)
    command.execute(notebook_id=notebook_id, size=size, links=links, files=files)

@notebooks_group.command("logs", help="List notebook logs")
@click.option(
    "--id",
    "notebook_id",
    required=True,
    cls=common.GradientOption,
)
@click.option(
    "--line",
    "line",
    required=False,
    default=0,
    cls=common.GradientOption,
)
@click.option(
    "--limit",
    "limit",
    required=False,
    default=10000,
    cls=common.GradientOption,
)
@click.option(
    "--follow",
    "follow",
    required=False,
    default=False,
    cls=common.GradientOption,
)
@api_key_option
@common.options_file
def list_logs(notebook_id, line, limit, follow, options_file, api_key=None):
    command = notebooks.NotebookLogsCommand(api_key=api_key)
    command.execute(notebook_id, line, limit, follow)



import mock
import pytest
from click.testing import CliRunner

from gradient.api_sdk.clients.http_client import default_headers
from gradient.cli import cli
from tests import example_responses, MockResponse

EXPECTED_HEADERS = default_headers.copy()
EXPECTED_HEADERS["ps_client_name"] = "gradient-cli"

EXPECTED_HEADERS_WITH_CHANGED_API_KEY = EXPECTED_HEADERS.copy()
EXPECTED_HEADERS_WITH_CHANGED_API_KEY["X-API-Key"] = "some_key"


class TestEntityAddTags(object):
    URL = "https://api.paperspace.io/entityTags/updateTags"

    COMMAND = [
        "tags", "add",
        "--tag", "test0",
        "--tag", "test1",
        "--tags", "test2,test3",
        "--id", "some_id",
    ]
    COMMAND_WITH_OPTIONS_FILE = ["tags", "add", "--optionsFile", ]  # path added in test

    TAGS_JSON = {
        "entity": "",
        "entityId": "some_id",
        "tags": ["test0", "test1", "test2", "test3"]
    }
    UPDATE_TAGS_RESPONSE_JSON_200 = example_responses.UPDATE_TAGS_RESPONSE
    EXPECTED_STDOUT = "Tags added to %s\n"

    @mock.patch("gradient.api_sdk.clients.http_client.requests.put")
    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
    @pytest.mark.parametrize(
        "entity_command, entity, result_entity",
        [
            ("deployments", "deployment", "deployment"),
            ("experiments", "experiment", "experiment"),
            ("experiments hyperparameters", "experiment", "hyperparameter"),
            ("jobs", "job", "job"),
            ("machines", "machine", "machine"),
            ("models", "mlModel", "ml model"),
            ("notebooks", "notebook", "notebook"),
            ("projects", "project", "project"),
        ]
    )
    def test_should_send_proper_data_and_success(self, get_patched, put_patched, entity_command, entity, result_entity):
        entity_command = entity_command.split(" ")
        command = entity_command + self.COMMAND

        tags_json = self.TAGS_JSON.copy()
        tags_json["entity"] = entity
        expected_result = self.EXPECTED_STDOUT % result_entity

        get_patched.return_value = MockResponse({}, 200, "fake content")
        put_patched.return_value = MockResponse(self.UPDATE_TAGS_RESPONSE_JSON_200, 200, "fake content")

        runner = CliRunner()
        result = runner.invoke(cli.cli, command)

        assert result.output == expected_result, result.exc_info
        put_patched.assert_called_once_with(
            self.URL,
            headers=EXPECTED_HEADERS,
            json=tags_json,
            params=None,
            data=None,
        )

        assert result.exit_code == 0

    @mock.patch("gradient.api_sdk.clients.http_client.requests.put")
    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
    @pytest.mark.parametrize(
        "entity_command, entity, result_entity",
        [
            ("deployments", "deployment", "deployment"),
            ("experiments", "experiment", "experiment"),
            ("experiments hyperparameters", "experiment", "hyperparameter"),
            ("jobs", "job", "job"),
            ("machines", "machine", "machine"),
            ("models", "mlModel", "ml model"),
            ("notebooks", "notebook", "notebook"),
            ("projects", "project", "project"),
        ]
    )
    def test_should_read_options_from_yaml_file(
            self, get_patched, put_patched, entity_tags_add_config_path, entity_command, entity, result_entity
    ):
        get_patched.return_value = MockResponse({}, 200, "fake content")
        put_patched.return_value = MockResponse(self.UPDATE_TAGS_RESPONSE_JSON_200, 200, "fake content")

        entity_command = entity_command.split(" ")
        command = entity_command + self.COMMAND_WITH_OPTIONS_FILE + [entity_tags_add_config_path]

        tags_json = self.TAGS_JSON.copy()
        tags_json["entity"] = entity
        expected_result = self.EXPECTED_STDOUT % result_entity

        runner = CliRunner()
        result = runner.invoke(cli.cli, command)

        put_patched.assert_called_once_with(
            self.URL,
            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
            json=tags_json,
            params=None,
            data=None,
        )

        assert result.output == expected_result, result.exc_info
        assert result.exit_code == 0


class TestEntityRemoveTags(object):
    URL = "https://api.paperspace.io/entityTags/updateTags"

    COMMAND = [
        "tags", "remove",
        "--tag", "test0",
        "--tags", "test2",
        "--id", "some_id",
    ]
    COMMAND_WITH_OPTIONS_FILE = ["tags", "remove", "--optionsFile", ]  # path added in test

    GET_TAGS_MOCK_RESPONSE = example_responses.GET_TAGS_RESPONSE

    TAGS_JSON = {
        "entity": "",
        "entityId": "some_id",
        "tags": ["test1", "test3"]
    }
    UPDATE_TAGS_RESPONSE_JSON_200 = example_responses.UPDATE_TAGS_RESPONSE
    EXPECTED_STDOUT = "Tags removed from %s\n"

    @mock.patch("gradient.api_sdk.clients.http_client.requests.put")
    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
    @pytest.mark.parametrize(
        "entity_command, entity, result_entity",
        [
            ("deployments", "deployment", "deployment"),
            ("experiments", "experiment", "experiment"),
            ("experiments hyperparameters", "experiment", "hyperparameter"),
            ("jobs", "job", "job"),
            ("machines", "machine", "machine"),
            ("models", "mlModel", "ml model"),
            ("notebooks", "notebook", "notebook"),
            ("projects", "project", "project"),
        ]
    )
    def test_should_send_proper_data_and_success(self, get_patched, put_patched, entity_command, entity, result_entity):
        entity_command = entity_command.split(" ")
        command = entity_command + self.COMMAND

        tags_json = self.TAGS_JSON.copy()
        tags_json["entity"] = entity
        expected_result = self.EXPECTED_STDOUT % result_entity

        get_patched.return_value = MockResponse(self.GET_TAGS_MOCK_RESPONSE, 200, "fake content")
        put_patched.return_value = MockResponse(self.UPDATE_TAGS_RESPONSE_JSON_200, 200, "fake content")

        runner = CliRunner()
        result = runner.invoke(cli.cli, command)

        assert result.output == expected_result, result.exc_info
        put_patched.assert_called_once_with(
            self.URL,
            headers=EXPECTED_HEADERS,
            json=tags_json,
            params=None,
            data=None,
        )

        assert result.exit_code == 0

    @mock.patch("gradient.api_sdk.clients.http_client.requests.put")
    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
    @pytest.mark.parametrize(
        "entity_command, entity, result_entity",
        [
            ("deployments", "deployment", "deployment"),
            ("experiments", "experiment", "experiment"),
            ("experiments hyperparameters", "experiment", "hyperparameter"),
            ("jobs", "job", "job"),
            ("machines", "machine", "machine"),
            ("models", "mlModel", "ml model"),
            ("notebooks", "notebook", "notebook"),
            ("projects", "project", "project"),
        ]
    )
    def test_should_read_options_from_yaml_file(
            self, get_patched, put_patched, entity_tags_remove_config_path, entity_command, entity, result_entity
    ):
        get_patched.return_value = MockResponse(self.GET_TAGS_MOCK_RESPONSE, 200, "fake content")
        put_patched.return_value = MockResponse(self.UPDATE_TAGS_RESPONSE_JSON_200, 200, "fake content")

        entity_command = entity_command.split(" ")
        command = entity_command + self.COMMAND_WITH_OPTIONS_FILE + [entity_tags_remove_config_path]

        tags_json = self.TAGS_JSON.copy()
        tags_json["entity"] = entity
        expected_result = self.EXPECTED_STDOUT % result_entity

        runner = CliRunner()
        result = runner.invoke(cli.cli, command)

        assert result.output == expected_result, result.exc_info
        put_patched.assert_called_once_with(
            self.URL,
            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
            json=tags_json,
            params=None,
            data=None,
        )

        assert result.exit_code == 0



from . import base_client
from .base_client import TagsSupportMixin
from .. import models, repositories


class HyperparameterJobsClient(TagsSupportMixin, base_client.BaseClient):
    entity = "experiment"

    def create(
            self,
            name,
            project_id,
            tuning_command,
            worker_container,
            worker_machine_type,
            worker_command,
            worker_count,
            worker_container_user=None,
            worker_registry_username=None,
            worker_registry_password=None,
            is_preemptible=False,
            ports=None,
            workspace_url=None,
            artifact_directory=None,
            cluster_id=None,
            experiment_env=None,
            trigger_event_id=None,
            model_type=None,
            model_path=None,
            dockerfile_path=None,
            hyperparameter_server_registry_username=None,
            hyperparameter_server_registry_password=None,
            hyperparameter_server_container=None,
            hyperparameter_server_container_user=None,
            hyperparameter_server_machine_type=None,
            working_directory=None,
            use_dockerfile=False,
            tags=None,
    ):
        """Create hyperparameter tuning job
        :param str name: Name of new experiment [required]
        :param str project_id: Project ID [required]
        :param str tuning_command: Tuning command [required]
        :param str worker_container: Worker container  [required]
        :param str worker_machine_type: Worker machine type  [required]
        :param str worker_command: Worker command  [required]
        :param int worker_count: Worker count  [required]
        :param str worker_container_user: Worker Container user
        :param str worker_registry_username: Worker registry username
        :param str worker_registry_password: Worker registry password
        :param bool is_preemptible: Flag: is preemptible
        :param str ports: Port to use in new experiment
        :param str workspace_url: Project git repository url
        :param str artifact_directory: Artifacts directory
        :param str cluster_id: Cluster ID
        :param dict experiment_env: Environment variables (in JSON)
        :param str trigger_event_id: GradientCI trigger event id
        :param str model_type: Model type
        :param str model_path: Model path
        :param str dockerfile_path: Path to dockerfile in project
        :param str hyperparameter_server_registry_username: Hyperparameter server registry username
        :param str hyperparameter_server_registry_password: Hyperparameter server registry password
        :param str hyperparameter_server_container: Hyperparameter server container
        :param str hyperparameter_server_container_user: Hyperparameter server container user
        :param str hyperparameter_server_machine_type: Hyperparameter server machine type
        :param str working_directory: Working directory for the experiment
        :param bool use_dockerfile: Flag: use dockerfile
        :param list[str] tags: List of tags

        :returns: ID of a new job
        :rtype: str
        """

        if not is_preemptible:
            is_preemptible = None

        if use_dockerfile is False:
            use_dockerfile = None

        hyperparameter = models.Hyperparameter(
            name=name,
            project_id=project_id,
            tuning_command=tuning_command,
            worker_container=worker_container,
            worker_container_user=worker_container_user,
            worker_machine_type=worker_machine_type,
            worker_command=worker_command,
            worker_count=worker_count,
            worker_registry_username=worker_registry_username,
            worker_registry_password=worker_registry_password,
            is_preemptible=is_preemptible,
            ports=ports,
            workspace_url=workspace_url,
            artifact_directory=artifact_directory,
            cluster_id=cluster_id,
            experiment_env=experiment_env,
            trigger_event_id=trigger_event_id,
            model_type=model_type,
            model_path=model_path,
            dockerfile_path=dockerfile_path,
            hyperparameter_server_machine_type=hyperparameter_server_machine_type,
            hyperparameter_server_registry_username=hyperparameter_server_registry_username,
            hyperparameter_server_registry_password=hyperparameter_server_registry_password,
            hyperparameter_server_container=hyperparameter_server_container,
            hyperparameter_server_container_user=hyperparameter_server_container_user,
            working_directory=working_directory,
            use_dockerfile=use_dockerfile,
        )

        repository = self.build_repository(repositories.CreateHyperparameterJob)
        handle = repository.create(hyperparameter)

        if tags:
            self.add_tags(entity_id=handle, tags=tags)

        return handle

    def run(
            self,
            name,
            project_id,
            tuning_command,
            worker_container,
            worker_machine_type,
            worker_command,
            worker_count,
            worker_registry_username=None,
            worker_registry_password=None,
            worker_container_user=None,
            is_preemptible=False,
            ports=None,
            workspace_url=None,
            artifact_directory=None,
            cluster_id=None,
            experiment_env=None,
            trigger_event_id=None,
            model_type=None,
            model_path=None,
            dockerfile_path=None,
            hyperparameter_server_registry_username=None,
            hyperparameter_server_registry_password=None,
            hyperparameter_server_container_user=None,
            hyperparameter_server_container=None,
            hyperparameter_server_machine_type=None,
            working_directory=None,
            use_dockerfile=False,
            tags=None,
    ):
        """Create and start hyperparameter tuning job

        :param str name: Name of new experiment  [required]
        :param str project_id: Project ID  [required]
        :param str tuning_command: Tuning command  [required]
        :param str worker_container: Worker container  [required]
        :param str worker_machine_type: Worker machine type  [required]
        :param str worker_command: Worker command  [required]
        :param int worker_count: Worker count  [required]
        :param str worker_container_user: Worker container user
        :param worker_registry_password: Worker registry password
        :param worker_registry_username: Worker registry username
        :param bool is_preemptible: Flag: is preemptible
        :param str ports: Port to use in new experiment
        :param str workspace_url: Project git repository url
        :param str artifact_directory: Artifacts directory
        :param str cluster_id: Cluster ID
        :param dict experiment_env: Environment variables (in JSON)
        :param str trigger_event_id: GradientCI trigger event id
        :param str model_type: Model type
        :param str model_path: Model path
        :param str dockerfile_path: Path to dockerfile
        :param str hyperparameter_server_registry_username: container registry username
        :param str hyperparameter_server_registry_password: container registry password
        :param str hyperparameter_server_container_user: hps container user
        :param str hyperparameter_server_container: hps container
        :param str hyperparameter_server_machine_type: hps machine type
        :param str working_directory: Working directory for the experiment
        :param bool use_dockerfile: Flag: use dockerfile
        :param list[str] tags: List of tags

        :returns: ID of a new job
        :rtype: str
        """

        if not is_preemptible:
            is_preemptible = None

        if use_dockerfile is False:
            use_dockerfile = None

        hyperparameter = models.Hyperparameter(
            name=name,
            project_id=project_id,
            tuning_command=tuning_command,
            worker_container=worker_container,
            worker_machine_type=worker_machine_type,
            worker_command=worker_command,
            worker_count=worker_count,
            worker_container_user=worker_container_user,
            worker_registry_username=worker_registry_username,
            worker_registry_password=worker_registry_password,
            is_preemptible=is_preemptible,
            ports=ports,
            workspace_url=workspace_url,
            artifact_directory=artifact_directory,
            cluster_id=cluster_id,
            experiment_env=experiment_env,
            trigger_event_id=trigger_event_id,
            model_type=model_type,
            model_path=model_path,
            dockerfile_path=dockerfile_path,
            hyperparameter_server_registry_username=hyperparameter_server_registry_username,
            hyperparameter_server_registry_password=hyperparameter_server_registry_password,
            hyperparameter_server_container_user=hyperparameter_server_container_user,
            hyperparameter_server_container=hyperparameter_server_container,
            hyperparameter_server_machine_type=hyperparameter_server_machine_type,
            working_directory=working_directory,
            use_dockerfile=use_dockerfile,
        )

        repository = self.build_repository(repositories.CreateAndStartHyperparameterJob)
        handle = repository.create(hyperparameter)

        if tags:
            self.add_tags(entity_id=handle, tags=tags)

        return handle

    def get(self, id):
        """Get Hyperparameter tuning job's instance

        :param str id: Hyperparameter job id

        :returns: instance of Hyperparameter
        :rtype: models.Hyperparameter
        """

        repository = self.build_repository(repositories.GetHyperparameterTuningJob)
        job = repository.get(id=id)
        return job

    def start(self, id):
        """Start existing hyperparameter tuning job

        :param str id: Hyperparameter job id
        :raises: exceptions.GradientSdkError
        """

        repository = self.build_repository(repositories.StartHyperparameterTuningJob)
        repository.start(id_=id)

    def list(self):
        """Get a list of hyperparameter tuning jobs

        :rtype: list[models.Hyperparameter]
        """
        repository = self.build_repository(repositories.ListHyperparameterJobs)
        experiments = repository.list()
        return experiments



import click

from gradient.cli import common
from gradient.cli.cli import cli
from gradient.commands import tensorboards as tensorboards_commands


@cli.group("tensorboards", help="Manage tensorboards", cls=common.ClickGroup)
def tensorboards_group():
    pass


@tensorboards_group.command("create", help="Create new tensorboard")
@click.option(
    "--experiment",
    "experiments",
    multiple=True,
    required=True,
    metavar="[<experiment ID>]",
    help="One or more experiment IDs [--experiment id1 --experiment id2 ...]",
    cls=common.GradientOption,
)
@click.option(
    "--image",
    "image",
    help="Tensorboard Container Image, by default its set to tensorflow/tensorflow:latest",
    cls=common.GradientOption,
)
@click.option(
    "--username",
    "username",
    help="Basic Auth Username",
    cls=common.GradientOption,
)
@click.option(
    "--password",
    "password",
    help="Basic Auth Password",
    cls=common.GradientOption,
)
# @click.option(
#     "--instanceType",
#     "instance_type",
#     help="Instance type",
#     cls=common.GradientOption,
# )
# @click.option(
#     "--instanceSize",
#     "instance_size",
#     help="Instance size",
#     cls=common.GradientOption,
# )
# @click.option(
#     "--instancesCount",
#     "instances_count",
#     type=int,
#     help="Instances count",
#     cls=common.GradientOption,
# )
@common.api_key_option
@common.options_file
def create_tensorboard(api_key, options_file, **kwargs):
    command = tensorboards_commands.CreateTensorboardCommand(api_key=api_key)
    command.execute(**kwargs)


@tensorboards_group.command("details", help="Show details of a tensorboard")
@click.option(
    "--id",
    "id",
    metavar="<tensorboard ID>",
    required=True,
    help="Tensorboard ID",
    cls=common.GradientOption,
)
@common.api_key_option
@common.options_file
def tensorboard_details(id, api_key, options_file):
    command = tensorboards_commands.GetTensorboardCommand(api_key=api_key)
    command.execute(id)


@tensorboards_group.command("list", help="Show list of tensorboards")
@common.api_key_option
@common.options_file
def list_tensorboards(api_key, options_file):
    command = tensorboards_commands.ListTensorboardsCommand(api_key=api_key)
    command.execute()


@tensorboards_group.command("add-experiments", help="Update tensorboard experiments")
@click.option(
    "--id",
    "tensorboard_id",
    metavar="<tensorboard ID>",
    required=True,
    help="Tensorboard ID",
    cls=common.GradientOption,
)
@click.option(
    "--experiment",
    "experiments",
    multiple=True,
    required=True,
    metavar="[<experiment ID>]",
    help="One or more experiment IDs [--experiment id1 --experiment id2 ...]",
    cls=common.GradientOption,
)
@common.api_key_option
@common.options_file
def add_experiments_to_tensorboard(tensorboard_id, experiments, api_key, options_file):
    command = tensorboards_commands.AddExperimentToTensorboard(api_key=api_key)
    command.execute(tensorboard_id, experiments)


@tensorboards_group.command("remove-experiments", help="Update tensorboard experiments")
@click.option(
    "--id",
    "id",
    required=True,
    metavar="<tensorboard ID>",
    help="Tensorboard ID",
    cls=common.GradientOption,
)
@click.option(
    "--experiment",
    "experiments",
    multiple=True,
    required=True,
    metavar="[<experiment ID>]",
    help="One or more experiment IDs [--experiment id1 --experiment id2 ...]",
    cls=common.GradientOption,
)
@common.api_key_option
@common.options_file
def remove_experiments_to_tensorboard(api_key, options_file, **kwargs):
    command = tensorboards_commands.RemoveExperimentToTensorboard(api_key=api_key)
    command.execute(**kwargs)


@tensorboards_group.command("delete", help="Delete tensorboard")
@click.option(
    "--id",
    "id",
    metavar="<tensorboard ID>",
    required=True,
    help="Tensorboard ID",
    cls=common.GradientOption,
)
@common.api_key_option
@common.options_file
def delete_tensorboard(api_key, options_file, **kwargs):
    command = tensorboards_commands.DeleteTensorboard(api_key=api_key)
    command.execute(**kwargs)
